# --
# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
# --

package Kernel::GenericInterface::Invoker::Mattermost::MattermostText;

use strict;
use warnings;
# For debug pourpose only
#use Data::Dumper qw(Dumper);

use Kernel::System::VariableCheck qw(IsString IsStringWithData);

our $ObjectManagerDisabled = 1;

=head1 NAME

Kernel::GenericInterface::Invoker::Mattermost::MattermostText - GenericInterface  Invoker backend

=head1 SYNOPSIS

=head1 PUBLIC INTERFACE

=over 4

=cut

=item new()

usually, you want to create an instance of this
by using Kernel::GenericInterface::Invoker->new();

=cut

sub new {
    my ( $Type, %Param ) = @_;

    # allocate new hash for object
    my $Self = {};
    bless( $Self, $Type );

    # check needed params
    if ( !$Param{DebuggerObject} ) {
        return {
            Success      => 0,
            ErrorMessage => "Got no DebuggerObject!"
        };
    }

    $Self->{DebuggerObject} = $Param{DebuggerObject};

    return $Self;
}

=item PrepareRequest()

prepare the invocation of the configured remote webservice.
This will just return the data that was passed to the function.

    my $Result = $InvokerObject->PrepareRequest(
        Data => {                               # data payload
            ...
        },
    );

    $Result = {
        Success         => 1,                   # 0 or 1
        ErrorMessage    => '',                  # in case of error
        Data            => {                    # data payload after Invoker
            ...
        },
    };

=cut

sub PrepareRequest {
    my ( $Self, %Param ) = @_;

#    For debug only
#    my $logfilename = '/opt/otrs/Custom/Kernel/GenericInterface/Invoker/Mattermost/mylog.txt';
#    open(my $fh, '>:encoding(UTF-8)', $logfilename) or die "Can not open '$logfilename' $!";
    
    
    # we need a TicketID
        if ( !IsStringWithData( $Param{Data}->{TicketID} ) ) {
        return $Self->{DebuggerObject}->Error( Summary => 'Got no TicketID' );
    }

    my %MattermostText = {};

    local $Kernel::OM = Kernel::System::ObjectManager->new();
    my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
    my $CustomerCompanyObject = $Kernel::OM->Get('Kernel::System::CustomerCompany');

    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
    my $Host = $ConfigObject->Get('HttpType') . '://';
    my $FQDN = $ConfigObject->Get('FQDN');
    if ( $FQDN ne 'yourhost.example.com' && gethostbyname($FQDN) ) {
             $Host .= $FQDN;
          }
    if ( !$Host && gethostbyname('localhost') ) {
             $Host .= 'localhost';
          }
    if ( !$Host ) {
           $Host .= '127.0.0.1';
          }

    my %Ticket = $TicketObject->TicketGet(
        TicketID      => $Param{Data}->{TicketID},
        DynamicFields => 0,         # Optional, default 0. To include the dynamic field values for this ticket on the return structure.
        UserID        => 1,
        Silent        => 0,         # Optional, default 0. To suppress the warning if the ticket does not exist.
        );

    my %CustomerCompany = $CustomerCompanyObject->CustomerCompanyGet(
        CustomerID => $Ticket{CustomerID} ,
            );
# Use it fot make additional field with "attachments" name to the post-request to Mattermost (incompatible with Telegam)
# See https://docs.mattermost.com/developer/message-attachments.html
# 
#    $MattermostText{Data}->{attachments}=[
#        {
#        'fallback'=>"New ticket from $CustomerCompany{CustomerCompanyName}",
#        'pretext'=> "New ticket from $CustomerCompany{CustomerCompanyName} into queue: $Ticket{Queue}", 
#        'color'=> '#7CD197',
#        'title'=> "Ticket #$Ticket{TicketNumber}",
#        'title_link'=>"$Host/otrs/index.pl?Action=AgentTicketZoom;TicketID=$Param{Data}->{TicketID}",
#        'text'=>"$Ticket{Title}"
#        }
#    ];

# Add field with "text" name to post-request to Mattermost for Telegram compatibility
# See https://docs.mattermost.com/developer/webhooks-incoming.html
#
    ($MattermostText{Data}->{text}=qq{
        #$Ticket{TicketNumber}
	Customer: $CustomerCompany{CustomerCompanyName}
	Queue: $Ticket{Queue}
	Priority: $Ticket{Priority}
	State: $Ticket{State}
	Owner: $Ticket{Owner}
	Responsible: $Ticket{Responsible}
	Changed : $Ticket{Changed}

        $Host/otrs/index.pl?Action=AgentTicketZoom;TicketID=$Param{Data}->{TicketID}
	}) =~ s/^[\t ]*//mg;

# Add field with "queue" name to post-request to Mattermost
# We will convert it to "channel" field in post-request by the "Simple" mapping in OTRS
#
    $MattermostText{Data}->{queue}=$Ticket{Queue};

#    Instead previus item you can hardcode channel by following
#    $MattermostText{Data}->{channel}="test1";


#    #Debug
#    my $dump = Data::Dumper->Dump([$ConfigObject], [qw(ConfigObject)]);
#    print $fh "\$MattermostText:\n$dump\n";



    
#    Close debug log file
#    close $fh;
    
    return {
        Success => 1,
        Data    => $MattermostText{Data},
    };
}

=item HandleResponse()

handle response data of the configured remote webservice.
This will just return the data that was passed to the function.

    my $Result = $InvokerObject->HandleResponse(
        ResponseSuccess      => 1,              # success status of the remote webservice
        ResponseErrorMessage => '',             # in case of webservice error
        Data => {                               # data payload
            ...
        },
    );

    $Result = {
        Success         => 1,                   # 0 or 1
        ErrorMessage    => '',                  # in case of error
        Data            => {                    # data payload after Invoker
            ...
        },
    };

=cut

sub HandleResponse {
    my ( $Self, %Param ) = @_;

    # if there was an error in the response, forward it
    if ( !$Param{ResponseSuccess} ) {
        return {
            Success      => 0,
            ErrorMessage => $Param{ResponseErrorMessage},
        };
    }

    if ( $Param{Data}->{ResponseContent} && $Param{Data}->{ResponseContent} =~ m{ReSchedule=1} ) {

        # ResponseContent has URI like params, convert them into a hash
        my %QueryParams = split /[&=]/, $Param{Data}->{ResponseContent};

        # unscape URI strings in query parameters
        for my $Param ( sort keys %QueryParams ) {
            $QueryParams{$Param} = URI::Escape::uri_unescape( $QueryParams{$Param} );
        }

        # fix ExecutrionTime param
        if ( $QueryParams{ExecutionTime} ) {
            $QueryParams{ExecutionTime} =~ s{(\d+)\+(\d+)}{$1 $2};
        }

        return {
            Success      => 0,
            ErrorMessage => 'Re-Scheduling...',
            Data         => \%QueryParams,
        };
    }

    return {
        Success => 1,
        Data    => $Param{Data},
    };

}

1;

=back

=head1 TERMS AND CONDITIONS

This software is part of the OTRS project (L<http://otrs.org/>).

This software comes with ABSOLUTELY NO WARRANTY. For details, see
the enclosed file COPYING for license information (AGPL). If you
did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.

=cut

